/*
  TITERM.C - Terminal Program for use with linkport.92p

  Uses code (a lot of it :-) from ti.c by dan@betterthan.northstar.k12.ak.us
  originally, modified (presumably) by davidell@ticalc.org.

  Modified/Written for DJGPP C by bryanr@flash.net.
  Highly optimized by timg@means.net.

  Note that I did not even try to silence -Wall. The source compiles, and
  that's all that counts. :-)

  0 - Both Red and White low - FREEZES CALC
  1 - Red line high
  2 - White line high
  3 - Both lines high (not used, normally)
*/

// Includes
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <pc.h>
#include <dos.h>

// You may need to tweak TIMAXTIME
#define TIMAXTIME 10000
#define lpt_base 0x378
#define lpt_out lpt_base
#define lpt_in	(lpt_base+1)

int init_ti(void);
int get_ti(void);
int put_ti(unsigned char data);

main() {
  int ret,stop=0,key;

  printf("Welcome to TI-92 Term. Hit ESC to quit.\n\n");
  fflush(stdout);
  init_ti();

  while( stop != 1 )
  {
    if ( kbhit() != 0 ) {
      key = getch();
      if ( key == 27 ) {
        stop=1;
        continue;
      }
      put_ti(key);
      printf("%c",key);
      if ( key == 13 ) {
         printf("\n");
         fflush(stdout);
         continue;
         }
      fflush(stdout);
      continue;
    }
  
    ret=get_ti();
    if ( ret == -1 ) continue;
    if ( ret == 13 ) {
       printf("\n");
       fflush(stdout);
       continue;
       }
    printf("%c",ret);
    fflush(stdout);
  }

  fflush(stdin);
  return 0;
}

// Initialize Link port
int init_ti(void) {
  outportb(lpt_out,3);
  return 0;
}

// Send a byte out to the TI-92
int put_ti(unsigned char data) {
  int bit,titime=0;
	
  for (bit=0; bit<8; bit++) {
    inportb(lpt_in);
    inportb(lpt_in);
    inportb(lpt_in);
    if (data&1) {
      outportb(lpt_out,2);
      while (inportb(lpt_in)&0x10) if(titime++>TIMAXTIME) return -1;
      outportb(lpt_out,3);
      while ((inportb(lpt_in)&0x10)==0x00) if(titime++>TIMAXTIME) return -1;
    } else {
      outportb(lpt_out, 1);
      while (inportb(lpt_in)&0x20) if(titime++>TIMAXTIME) return -1;
      outportb(lpt_out,3);
      while ((inportb(lpt_in)&0x20)==0x00) if(titime++>TIMAXTIME) return -1;
    }
    data>>=1;
  }
  return 0;
}

// Receive a byte from the TI-92
int get_ti(void) {
  int bit,titime=0;
  unsigned char data=0, v;
  
  for (bit=0; bit<8; bit++) {
    while ((v=inportb(lpt_in)&0x30) == 0x30) if(titime++>TIMAXTIME) return -1;
    if (v==0x10) {
      data=(data>>1)|0x80;
      outportb(lpt_out,1);
      while ((inportb(lpt_in)&0x20)==0x00) if(titime++>TIMAXTIME) return -1;
      outportb(lpt_out,3);
    } else {
      data=data>>1;
      outportb(lpt_out,2);
      while ((inportb(lpt_in)&0x10)==0x00) if(titime++>TIMAXTIME) return -1;
      outportb(lpt_out,3);
    }
    inportb(lpt_in);
    inportb(lpt_in);
    inportb(lpt_in);
  }
  return data;
}
